#!/bin/sh

set -e

# This script is meant to be ran on Ubuntu 20.04 LTS amd64 only
DISTRO_ID=$(lsb_release -is 2>/dev/null || true)
RELEASE=$(lsb_release -rs 2>/dev/null || true)
if [ ! "x$DISTRO_ID" = "xUbuntu" ] || [ "x$RELEASE" != "x20.04" ]; then
	printf "Error: unsupported distribution (Ubuntu 20.04 LTS amd64 only)\n" >&2
	exit 1
fi

REBOOT_REQ=0
LOG_FILE="/var/log/install-xdr-nsva.log"
SCRIPT_VERSION="1.1"

usage ()
{
	cat >&2 <<EOF

Warning: this script applies a security hardenings policy that restricts SSH access to this machine.
To avoid getting locked out, make sure to either:
1) set a usable password for the root user
or
2) provide a list of users to receive SSH access to this machine:

Usage: $0 --users <list of users separated by space>

EOF
}

check_users ()
{
	EUID=$(id -u)
	if [ $EUID -ne 0 ]; then
		printf "Error: this script must be ran as root\n" >&2
		exit 1
	fi

	# Make sure we can connect via SSH after security hardenings are applied
	if [ "$#" -eq 0 ]; then
		ROOT_PASSWD=$(passwd --status root | awk '{ print $2 }' || true)
		if [ "x$ROOT_PASSWD" != "xP" ]; then
			printf "Error: provide a list of users to receive SSH access or set a usable root password\n" >&2
			printf "For more information, run the following command: $0 --help\n" >&2
			exit 1
		fi
	fi
}

# Update the netplan config files to use "eth0"/"eth1"
# Otherwise, the interfaces won't have an IP address after reboot
netplan_pre_install ()
{
	netdev_list=$(ls /sys/class/net | awk '{ print $1 }')
	index=0

	for netdev in $netdev_list; do
		[ "$netdev" = "lo" ] && continue

		eth_name="eth${index}"
		find "/etc/netplan" -type f -name "*.yaml" | xargs sed -i "s/${netdev}/${eth_name}/g" || true
		index=$(($index + 1))
	done

}

network_config_pre_install ()
{
	netplan_chk=$(service systemd-networkd status | grep -c "active (running)" || true)
	net_mgr_chk=$(service network-manager status | grep -c "active (running)" || true)

	if [ $netplan_chk -ne 0 ] || [ $net_mgr_chk -ne 0 ]; then
		netplan_pre_install
	else
		# If the machine is already using ifupdown, it should be enough to just generate a new config file
		if dpkg -s ifupdown > /dev/null 2>&1; then
			generate_interfaces_file
		else
			# Maybe the machine is using some other service to handle networking??

			printf "Error: the network interfaces need to be named eth0 and eth1\n" >&2
			exit 1
		fi
	fi

	sed -i 's/^GRUB_CMDLINE_LINUX_DEFAULT=.*/GRUB_CMDLINE_LINUX_DEFAULT=\"netcfg\/do_not_use_netplan=true net.ifnames=0 biosdevname=0\"/g' "/etc/default/grub" || true
	update-grub || true

	REBOOT_REQ=1
}

generate_interfaces_file ()
{
	cat > "/etc/network/interfaces" << EOF
auto lo
iface lo inet loopback

auto eth0
iface eth0 inet dhcp

auto eth1
iface eth1 inet manual
up ip link set eth1 promisc on
EOF
}

# Add sysctl for network hardening
generate_hardening_sysctl ()
{
	cat > "/etc/sysctl.d/60-bd.conf" << EOF
# Ensure source routed packets are not accepted
net.ipv4.conf.all.accept_source_route = 0
net.ipv4.conf.default.accept_source_route = 0
# Ensure IP forwarding is disabled
net.ipv4.ip_forward = 0
# Ensure packet redirect sending is disabled
net.ipv4.conf.all.send_redirects = 0
net.ipv4.conf.default.send_redirects = 0

# Ensure suspicious packets are logged
# Disabled due to excessive logging because of ipfix data
# net.ipv4.conf.all.log_martians = 1
# net.ipv4.conf.default.log_martians = 1
# Ensure broadcast ICMP requests are ignored
net.ipv4.icmp_echo_ignore_broadcasts = 1
# Ensure bogus ICMP responses are ignored
net.ipv4.icmp_ignore_bogus_error_responses = 1
# Ensure Reverse Path Filtering is enabled
net.ipv4.conf.all.rp_filter = 1
net.ipv4.conf.default.rp_filter = 1
# Ensure TCP SYN Cookies is enabled
net.ipv4.tcp_syncookies = 1
# Ensure IPv6 router advertisements are not accepted
net.ipv6.conf.all.accept_ra = 0
net.ipv6.conf.default.accept_ra = 0
# Ensure IPv6 redirects are not accepted
net.ipv6.conf.all.accept_redirects = 0
net.ipv6.conf.default.accept_redirects = 0

EOF

	sysctl --system || true
}

# This script requires two network interfaces
check_network ()
{
	netdev_list_count=$(ls -A /sys/class/net | wc -l)

	if [ $netdev_list_count -lt 3 ]; then
		printf "Error: two network interfaces are required\n" >&2
		exit 1
	fi

	if ! grep -q "eth0" "/proc/net/dev" || ! grep -q "eth1" "/proc/net/dev"; then
		network_config_pre_install
	fi
}

config_apt ()
{
	# Move the Ubuntu repository into its own sources.list instance
	[ -f /etc/apt/sources.list.d/ubuntu.list ] || \
		cp -f /etc/apt/sources.list /etc/apt/sources.list.d/ubuntu.list

	# Clear the default sources.list
	> /etc/apt/sources.list

	# Disable the automatic upgrades (see dpkg-reconfigure unattended-upgrades)
	[ -f /etc/apt/apt.conf.d/20auto-upgrades ] && \
		cp -f /usr/share/unattended-upgrades/20auto-upgrades-disabled \
			/etc/apt/apt.conf.d/20auto-upgrades

	printf "deb https://download.bitdefender.com/repos/deb-hydra-unified-2004/ bitdefender non-free\n" >/etc/apt/sources.list.d/bitdefender.list
	printf "deb https://download.bitdefender.com/repos/deb-sva-2004-slow/ bitdefender non-free\n" >>/etc/apt/sources.list.d/bitdefender.list

	# Remove the obsolete i386 architecture
	OLD_ARCH=$(dpkg --print-foreign-architectures)
	[ "x$OLD_ARCH" = "xi386" ] && dpkg --remove-architecture "$OLD_ARCH"

	dpkg -s curl > /dev/null 2>&1 || apt-get -y install curl

	# Import the Bitdefender public repository key
	curl -sS https://download.bitdefender.com/repos/gzrepos.key.asc | apt-key add -

	# Update the package list
	apt-get update

	# Prevent debconf from asking any questions and just use the defaults
	DEBIAN_FRONTEND=noninteractive
	export DEBIAN_FRONTEND
}

config_network ()
{
	dpkg -s ifupdown resolvconf net-tools >/dev/null 2>&1 || \
		apt-get -y install ifupdown resolvconf net-tools

	if [ ! -x "/usr/sbin/ifup" ]; then
		printf "Error: unable to install ifupdown package\n" >&2
		exit 1
	fi

	# Disable netplan/NetworkManager network services
	netplan_chk=$(service systemd-networkd status | grep -c "active (running)" || true)
	net_mgr_chk=$(service network-manager status | grep -c "active (running)" || true)
	if [ $netplan_chk -ne 0 ] || [ $net_mgr_chk -ne 0 ]; then

		# Create new network config file
		if ! grep -q "eth0" "/etc/network/interfaces"; then
			generate_interfaces_file
		fi

		# Get rid of unneeded netplan services & packages
		systemctl mask systemd-networkd systemd-networkd.socket || true

		mkdir -p "/etc/systemd/system/systemd-networkd-wait-online.service.d" || true
		cat > "/etc/systemd/system/systemd-networkd-wait-online.service.d/override.conf" << EOF
[Service]
ExecStart=
ExecStart=/usr/lib/systemd/systemd-networkd-wait-online --any
EOF

		# Get rid of unneeded NetworkManager services & packages
		systemctl mask NetworkManager.service || true
		systemctl mask NetworkManager-wait-online.service || true
		systemctl mask NetworkManager-dispatcher.service || true
		systemctl mask network-manager.service || true

		REBOOT_REQ=1
	fi

	# Check if systemd-resolved is used for DNS
	resolved_chk=$(service systemd-resolved status | grep -c "active (running)" || true)
	if [ $resolved_chk -ne 0 ]; then
		systemctl enable resolvconf
		systemctl mask systemd-resolved

		# The resolved script for DHCP doesn't check correctly if systemd-resolved is being used
		# so DHCP no longer generates resolv.conf for non-static configurations as a result
		if dpkg-divert --test --divert /tmp/resolved --rename /etc/dhcp/dhclient-enter-hooks.d/resolved > /dev/null 2>&1; then
			dpkg-divert --divert /tmp/resolved --rename /etc/dhcp/dhclient-enter-hooks.d/resolved
		fi

		# Clear old DNS entries left by systemd-resolved
		> "/run/resolvconf/interface/systemd-resolved" || true
		> "/run/resolvconf/interface/original.resolvconf" || true
		> "/run/systemd/resolve/stub-resolv.conf" || true

		if [ ! -f "/etc/resolv.conf" ] || [ -L "/etc/resolv.conf" ]; then
			ln -sf "/run/resolvconf/resolv.conf" "/etc/resolv.conf" || true
		fi

		REBOOT_REQ=1
	fi

	if [ ! -f "/etc/sysctl.d/60-bd.conf" ]; then
		generate_hardening_sysctl
	fi

	# Restart interfaces
	if [ $REBOOT_REQ -ne 0 ]; then

		promisc_netdev=""
		if grep "eth0" "/etc/network/interfaces" | grep -q "promisc"; then
			promisc_netdev="eth0"
		fi
		if grep "eth1" "/etc/network/interfaces" | grep -q "promisc"; then
			promisc_netdev="eth1"
		fi

		if [ "x$promisc_netdev" != "x" ]; then
			# Clear any old DNS entries left by DHCP
			if [ -f "/run/resolvconf/interface/$promisc_netdev.dhclient" ]; then
				> "/run/resolvconf/interface/$promisc_netdev.dhclient" || true
			fi

			# Clear any DHCP leases left for the promisc interface
			rm -f "/var/lib/dhcp/dhclient.$promisc_netdev.leases" || true
		fi

		exec > "/dev/tty"

		printf "\nThe script requires a reboot before proceeding with the installation, please rerun the script after."
		printf "\nWarning: the IP address may change after reboot."
		printf "\nReboot now (yes/no)? "

		read user_answer
		user_answer=$(echo "$user_answer" | awk '{ print tolower($0) }')

		if [ "$user_answer" = "yes" ] || [ "$user_answer" = "y" ]; then
			reboot
		else
			return 1
		fi
	fi
}

install_pkg ()
{
	# Install ghostr packages
	dpkg -s ghostr-brain >/dev/null 2>&1 || \
		apt-get -y install ghostr-brain

	dpkg -s ghostr-probe >/dev/null 2>&1 || \
		apt-get -y install ghostr-probe

	# Install the XDR NSVA package if not already present
	dpkg -s bitdefender-xdr-nsva >/dev/null 2>&1 || \
		apt-get -y install bitdefender-xdr-nsva
}

config_ssh ()
{
	# Enable SSH access for the specified users
	if [ "$#" -gt 0 ]; then
		SSH_USERS=$(echo "$@" | tr -d '\n\r')
		SSH_CONFIG="/etc/ssh/sshd_config.d/bitdefender.conf"

		sed -i "s%AllowUsers\(.*\)$%AllowUsers\1 ${SSH_USERS}%g" "$SSH_CONFIG" || true
		service ssh restart >/dev/null 2>&1 || true
	fi
}

if [ "$#" -gt 0 ]; then
	OPTION="$1"
	case "$OPTION" in
		"--help") usage; exit 1;;
		"--users") shift 1;;
		*) printf "Error: invalid arguments\n"; usage; exit 1;;
	esac
fi

check_users "$@"

printf "\nThe logs can be found here: $LOG_FILE\n"
printf "\nThe script will install the Bitdefender XDR Network Security Virtual Appliance, please wait ... "
exec >> "$LOG_FILE" 2>&1
printf "\nScript version: $SCRIPT_VERSION\n"

check_network
config_apt
config_network
install_pkg
config_ssh "$@"

exec > "/dev/tty"

/opt/bitdefender/bin/sva_setup.sh

printf "\n"
printf "The installation is now complete.\n"
